<?php
class WPJAM_API{
	public static function __callStatic($method, $args){
		$function	= 'wpjam_'.$method;

		if(function_exists($function)){
			return call_user_func($function, ...$args);
		}
	}

	public static function get_apis(){	// 兼容
		return WPJAM_JSON::get_registereds();
	}
}

class WPJAM_Route{
	public static function init(){
		$GLOBALS['wp']->add_query_var('module');
		$GLOBALS['wp']->add_query_var('action');
		$GLOBALS['wp']->add_query_var('term_id');

		$GLOBALS['wpjam_grant']	= new WPJAM_Grant();

		wpjam_register('route_module', 'json',	['callback'=>['WPJAM_JSON', 'module']]);
		wpjam_register('route_module', 'txt',	['callback'=>['WPJAM_Verify_TXT', 'module']]);

		add_rewrite_rule($GLOBALS['wp_rewrite']->root.'api/([^/]+)/(.*?)\.json?$',	'index.php?module=json&action=mag.$matches[1].$matches[2]', 'top');
		add_rewrite_rule($GLOBALS['wp_rewrite']->root.'api/([^/]+)\.json?$',		'index.php?module=json&action=$matches[1]', 'top');

		add_filter('root_rewrite_rules',	['WPJAM_Verify_TXT', 'filter_root_rewrite_rules']);

		add_action('wpjam_api',	['WPJAM_JSON', 'register_default']);
	}

	public static function parse_tax_query($taxonomy, $term_id){
		if($term_id == 'none'){
			return ['taxonomy'=>$taxonomy,	'field'=>'term_id',	'operator'=>'NOT EXISTS'];
		}else{
			return ['taxonomy'=>$taxonomy,	'field'=>'term_id',	'terms'=>[$term_id]];
		}
	}

	public static function parse_query_vars($query_vars){
		$tax_query	= $query_vars['tax_query'] ?? [];
		$date_query	= $query_vars['date_query'] ?? [];

		$taxonomies	= array_values(get_taxonomies(['_builtin'=>false]));

		foreach(array_merge($taxonomies, ['category', 'post_tag']) as $taxonomy){
			$query_key	= wpjam_get_taxonomy_query_key($taxonomy);
			$term_id	= wpjam_array_pull($query_vars, $query_key);

			if($term_id){
				if($taxonomy == 'category' && $term_id != 'none'){
					$query_vars[$query_key]	= $term_id;
				}else{
					$tax_query[]	= self::parse_tax_query($taxonomy, $term_id);
				}
			}
		}

		if(!empty($query_vars['taxonomy']) && empty($query_vars['term'])){
			$term_id	= wpjam_array_pull($query_vars, 'term_id');

			if($term_id){
				if(is_numeric($term_id)){
					$taxonomy		= wpjam_array_pull($query_vars, 'taxonomy');
					$tax_query[]	= self::parse_tax_query($taxonomy, $term_id);
				}else{
					$query_vars['term']	= $term_id;
				}
			}
		}

		foreach(['cursor'=>'before', 'since'=>'after'] as $key => $query_key){
			$value	= wpjam_array_pull($query_vars, $key);

			if($value){
				$date_query[]	= [
					$query_key => get_date_from_gmt(date('Y-m-d H:i:s', $value))
				];
			}
		}

		if($tax_query){
			$query_vars['tax_query']	= $tax_query;
		}

		if($date_query){
			$query_vars['date_query']	= $date_query;
		}

		return $query_vars;
	}

	public static function on_parse_request($wp){
		$wp->query_vars	= self::parse_query_vars($wp->query_vars);
	}

	public static function on_send_headers($wp){
		$module	= $wp->query_vars['module'] ?? '';
		$action	= $wp->query_vars['action'] ?? '';

		if($module){
			$object	= wpjam_get_registered_object('route_module', $module);

			if($object){
				call_user_func($object->callback, $action, $module);
			}

			remove_action('template_redirect',	'redirect_canonical');

			add_filter('template_include',		[self::class, 'filter_template_include']);
		}
	}

	public static function filter_template_include(){
		$module		= get_query_var('module');
		$action		= get_query_var('action');

		$template	= $action ? $action.'.php' : 'index.php';
		$template	= STYLESHEETPATH.'/template/'.$module.'/'.$template;
		$template	= apply_filters('wpjam_template', $template, $module, $action);

		return is_file($template) ? $template : wp_die('路由错误！');
	}

	public static function filter_determine_current_user($user_id){
		if(empty($user_id)){
			$wpjam_user	= wpjam_get_current_user();

			if($wpjam_user && !empty($wpjam_user['user_id'])){
				return $wpjam_user['user_id'];
			}
		}

		return $user_id;
	}

	public static function filter_current_commenter($commenter){
		if(empty($commenter['comment_author_email'])){
			$wpjam_user	= wpjam_get_current_user();

			if($wpjam_user && !empty($wpjam_user['user_email'])){
				$commenter['comment_author_email']	= $wpjam_user['user_email'];
				$commenter['comment_author']		= $wpjam_user['nickname'];
			}
		}

		return $commenter;
	}
}

class WPJAM_JSON extends WPJAM_Register{
	private $response;

	public function validate(){
		if(!isset($_GET['access_token']) && is_super_admin()){
			return;
		}
		
		$appid	= wpjam_get_parameter('appid');

		if($this->grant){
			$token	= wpjam_get_parameter('access_token', ['required'=>true]);
			$item 	= $GLOBALS['wpjam_grant']->validate_token($token);

			if(is_wp_error($item)){
				wpjam_send_json($item);
			}

			$appid	= $item['appid'];
		}

		$times	= self::cache_get($appid, $this->name);

		if($this->quota && $times > $this->quota){
			wpjam_send_json(['errcode'=>'api_exceed_quota', 'errmsg'=>'API 调用次数超限']);
		}

		self::cache_set($appid, $this->name, $times+1);
	}

	public function response(){
		$current_user	= wpjam_get_current_user($this->auth);

		if(is_wp_error($current_user)){
			wpjam_send_json($current_user);
		}

		$this->response	= [
			'errcode'		=> 0,
			'current_user'	=> $current_user
		];

		if($_SERVER['REQUEST_METHOD'] != 'POST'){
			foreach(['page_title', 'share_title', 'share_image'] as $key){
				$this->response[$key]	= (string)$this->$key;
			}
		}

		if($this->modules){
			$modules	= $this->modules;
			$modules	= wp_is_numeric_array($modules) ? $modules : [$modules];

			foreach($modules as $module){
				$module	= wp_parse_args($module, ['type'=>'', 'args'=>[]]);				
				$object	= new WPJAM_JSON_Module($module['type'], $module['args']);
				$result	= $object->parse();

				$this->merge_result($result);
			}
		}else{
			$result	= null;

			if($this->callback){
				if(is_callable($this->callback)){
					$result	= call_user_func($this->callback, $this->args, $this->name);
				}
			}elseif($this->template){
				if(is_file($this->template)){
					$result	= include $this->template;
				}
			}else{
				$result	= $this->args;
			}

			$this->merge_result($result);
		}

		$response	= apply_filters('wpjam_json', $this->response, $this->args, $this->name);

		if($_SERVER['REQUEST_METHOD'] != 'POST'){
			if(empty($response['page_title'])){
				$response['page_title']		= html_entity_decode(wp_get_document_title());
			}

			if(empty($response['share_title'])){
				$response['share_title']	= $response['page_title'];
			}

			if(!empty($response['share_image'])){
				$response['share_image']	= wpjam_get_thumbnail($response['share_image'], '500x400');
			}
		}

		wpjam_send_json($response);
	}

	private function merge_result($result){
		if(is_wp_error($result)){
			wpjam_send_json($result);
		}elseif(is_array($result)){
			$except	= [];

			foreach(['page_title', 'share_title', 'share_image'] as $key){
				if(!empty($this->response[$key]) && isset($result[$key])){
					$except[]	= $key;
				}
			}

			if($except){
				$result	= wpjam_array_except($result, $except);
			}

			$this->response	= array_merge($this->response, $result);
		}
	}

	public static function module($action){
		if(!wpjam_doing_debug()){
			self::send_headers();
		}

		if(!str_starts_with($action, 'mag.')){
			return;
		}

		$name	= self::parse_name($action);

		do_action('wpjam_api', $name);

		$object	= self::get($name);

		if(!$object){
			wpjam_send_json(['errcode'=>'api_not_defined',	'errmsg'=>'接口未定义！']);
		}

		$object->validate();
		$object->response();
	}

	public static function send_headers(){
		header('X-Content-Type-Options: nosniff');

		if($origin	= get_http_origin()){
			// Requests from file:// and data: URLs send "Origin: null"
			if('null' !== $origin){
				$origin	= esc_url_raw($origin);
			}

			@header('Access-Control-Allow-Origin: ' . $origin);
			@header('Access-Control-Allow-Methods: GET, POST');
			@header('Access-Control-Allow-Credentials: true');
			@header('Access-Control-Allow-Headers: Authorization, Content-Type');
			@header('Vary: Origin');

			if('OPTIONS' === $_SERVER['REQUEST_METHOD']){
				exit;
			}
		}

		if('OPTIONS' === $_SERVER['REQUEST_METHOD']){
			status_header(403);
			exit;
		}

		$type	= wp_is_jsonp_request() ? 'javascript' : 'json';

		@header('Content-Type: application/'.$type.'; charset='.get_option('blog_charset'));
	}

	public static function parse_name($action){
		return str_replace(['mag.','/'], ['','.'], $action);
	}

	public static function register_default($json){
		if(self::get($json)){
			return;
		}

		if($json == 'post.list'){
			$modules	= [];
			$post_type	= wpjam_get_parameter('post_type');

			if($post_type && is_string($post_type)){
				foreach(get_object_taxonomies($post_type, 'objects') as $taxonomy => $tax_obj){
					if($tax_obj->hierarchical && $tax_obj->public){
						$modules[]	= ['type'=>'taxonomy',	'args'=>['taxonomy'=>$taxonomy, 'hide_empty'=>0]];
					}
				}
			}

			$modules[]	= [
				'type'	=> 'post_type',
				'args'	=> ['post_type'=>$post_type, 'action'=>'list', 'posts_per_page'=>10, 'output'=>'posts']
			];

			self::register($json,	['modules'=>$modules]);
		}elseif($json == 'post.get'){
			self::register($json,	['modules'=>['type'=>'post_type',	'args'=>['action'=>'get', 'output'=>'post']]]);
		}elseif($json == 'media.upload'){
			self::register($json,	['modules'=>['type'=>'media',	'args'=>['media'=>'media']]]);
		}elseif($json == 'token.grant'){
			self::register($json,	['modules'=>['type'=>'token'],	'quota'=>1000]);
		}elseif($json == 'token.validate'){
			self::register($json,	['quota'=>10,	'grant'=>true]);
		}
	}

	public static function __callStatic($method, $args){
		if(in_array($method, ['cache_get', 'cache_set', 'cache_delete'])){
			$cg_obj	= WPJAM_Cache_Group::get_instance('wpjam_api_times');

			$appid	= $args[0];
			$today	= date('Y-m-d', current_time('timestamp'));
			$appid	= $appid ? $appid.':'.$today : $today;
			$json	= $args[1] ?? '';
			$caches	= $cg_obj->cache_get($appid) ?: [];

			if($method == 'cache_get'){
				if($json){
					return $caches[$json] ?? 0;
				}else{
					return $caches;
				}
			}elseif($method == 'cache_set'){
				$caches[$json] = $args[2];

				return $cg_obj->cache_set($appid, $caches);
			}elseif($method == 'cache_delete'){
				if($json){
					unset($caches[$json]);

					return $cg_obj->cache_set($appid, $caches);
				}else{
					return $cg_obj->cache_delete($appid);
				}
			}
		}elseif(in_array($method, ['parse_post_list_module', 'parse_post_get_module'])){
			$args	= $args[0] ?? [];
			$action	= str_replace(['parse_post_', '_module'], '', $method);

			return (new WPJAM_JSON_Module('post_type', array_merge($args, ['action'=>$action])))->parse();
		}
	}
}

class WPJAM_JSON_Module{
	protected $type;
	protected $args;

	public function __construct($type, $args=[]){
		if(!is_array($args)){
			$args	= wpjam_parse_shortcode_attr(stripslashes_deep($args), 'module');
		}

		$this->type	= $type;
		$this->args	= $args;
	}

	public function __get($name){
		return $this->args[$name] ?? null;
	}

	public function __isset($name){
		return isset($this->args[$name]);
	}

	public function parse(){
		$method	= 'parse_'.$this->type;

		if(method_exists($this, $method)){
			return call_user_func([$this, $method]);
		}else{
			return $this->args;
		}
	}

	public function parse_post_type(){
		$action	= wpjam_array_pull($this->args, 'action');

		if(!$action){
			wpjam_send_json(['errcode'=>'empty_action',	'errmsg'=>'没有设置 action']);
		}

		$wp	= $GLOBALS['wp'];

		if(isset($wp->raw_query_vars)){
			$wp->query_vars		= $wp->raw_query_vars;
		}else{
			$wp->raw_query_vars	= $wp->query_vars;
		}

		if($action == 'list'){
			return $this->parse_post_list();
		}elseif($action == 'get'){
			return $this->parse_post_get();
		}elseif($action == 'upload'){
			return $this->parse_media();
		}
	}

	/* 规则：
	** 1. 分成主的查询和子查询（$query_args['sub']=1）
	** 2. 主查询支持 $_GET 参数 和 $_GET 参数 mapping
	** 3. 子查询（sub）只支持 $query_args 参数
	** 4. 主查询返回 next_cursor 和 total_pages，current_page，子查询（sub）没有
	** 5. $_GET 参数只适用于 post.list
	** 6. term.list 只能用 $_GET 参数 mapping 来传递参数
	*/
	public function parse_post_list(){
		$output	= wpjam_array_pull($this->args, 'output');
		$sub	= wpjam_array_pull($this->args, 'sub');

		$is_main_query	= !$sub;	// 子查询不支持 $_GET 参数，置空之前要把原始的查询参数存起来

		if($is_main_query){
			$wp	= $GLOBALS['wp'];

			foreach($this->args as $query_key => $query_valeu){
				$wp->set_query_var($query_key, $query_valeu);
			}

			if($posts_per_page = (int)wpjam_get_parameter('posts_per_page')){
				$wp->set_query_var('posts_per_page', ($posts_per_page > 20 ? 20 : $posts_per_page));
			}

			if($offset = (int)wpjam_get_parameter('offset')){
				$wp->set_query_var('offset', $offset);
			}

			$orderby	= $wp->query_vars['orderby'] ?? 'date';
			$paged		= $wp->query_vars['paged'] ?? null;
			$use_cursor	= (empty($paged) && is_null(wpjam_get_parameter('s')) && !is_array($orderby) && in_array($orderby, ['date', 'post_date']));

			if($use_cursor){
				if($cursor = (int)wpjam_get_parameter('cursor')){
					$wp->set_query_var('cursor', $cursor);
					$wp->set_query_var('ignore_sticky_posts', true);
				}

				if($since = (int)wpjam_get_parameter('since')){
					$wp->set_query_var('since', $since);
					$wp->set_query_var('ignore_sticky_posts', true);
				}
			}

			$post_type	= $wp->query_vars['post_type'] ?? '';
			$taxonomies	= $post_type ? get_object_taxonomies($post_type) : get_taxonomies(['public'=>true]);
			$taxonomies	= array_diff($taxonomies, ['post_format']);

			foreach($taxonomies as $taxonomy){	// taxonomy 参数处理，同时支持 $_GET 和 $query_vars 参数
				$query_key	= wpjam_get_taxonomy_query_key($taxonomy);

				if($taxonomy == 'category'){
					if(empty($wp->query_vars['cat'])){
						foreach(['category_id', 'cat_id'] as $cat_key){
							$term_id	= (int)wpjam_get_parameter($cat_key);

							if($term_id){
								$wp->set_query_var($query_key, $term_id);
								break;
							}
						}
					}
				}else{
					$term_id	= (int)wpjam_get_parameter($query_key);

					if($term_id){
						$wp->set_query_var($query_key, $term_id);
					}
				}
			}

			$term_id	= (int)wpjam_get_parameter('term_id');
			$taxonomy	= wpjam_get_parameter('taxonomy');

			if($term_id && $taxonomy){
				$wp->set_query_var('term_id', $term_id);
				$wp->set_query_var('taxonomy', $taxonomy);
			}

			do_action('parse_request', $wp);

			$wp->query_posts();

			$wp_query	= $GLOBALS['wp_query'];
		}else{
			$post_type	= $this->post_type;
			$query_vars	= wpjam_parse_query_vars($this->args);
			$wp_query	= new WP_Query($query_vars);
		}

		if(empty($output)){
			if($post_type && is_string($post_type)){
				$output	= wpjam_get_post_type_setting($post_type, 'plural') ?: $post_type.'s';
			}else{
				$output	= 'posts';
			}
		}

		$_posts = [];

		while($wp_query->have_posts()){
			$wp_query->the_post();

			$_posts[]	= wpjam_get_post(get_the_ID(), $this->args);
		}

		$posts_json = [];

		if($is_main_query){
			if(is_category() || is_tag() || is_tax()){
				if($current_term = get_queried_object()){
					$taxonomy		= $current_term->taxonomy;
					$current_term	= wpjam_get_term($current_term, $taxonomy);

					$posts_json['current_taxonomy']		= $taxonomy;
					$posts_json['current_'.$taxonomy]	= $current_term;
				}else{
					$posts_json['current_taxonomy']		= null;
				}
			}elseif(is_author()){
				if($author = $wp_query->get('author')){
					$posts_json['current_author']	= WPJAM_User::get_instance($author)->parse_for_json();
				}else{
					$posts_json['current_author']	= null;
				}
			}

			$posts_json['total']		= (int)$wp_query->found_posts;
			$posts_json['total_pages']	= (int)$wp_query->max_num_pages;
			$posts_json['current_page']	= (int)($wp_query->get('paged') ?: 1);

			if($use_cursor){
				$posts_json['next_cursor']	= ($_posts && $wp_query->max_num_pages > 1) ? end($_posts)['timestamp'] : 0;
			}

			$posts_json['page_title']	= $posts_json['share_title'] = html_entity_decode(wp_get_document_title());
		}

		$posts_json[$output]	= $_posts;

		return apply_filters('wpjam_posts_json', $posts_json, $wp_query, $output);
	}

	public function parse_post_get(){
		global $wp, $wp_query;

		$post_id	= $this->id ?? (int)wpjam_get_parameter('id');
		$post_type	= $this->post_type ?? wpjam_get_parameter('post_type',	['default'=>'any']);

		if($post_type != 'any'){
			if(!post_type_exists($post_type)){
				wpjam_send_json(['errcode'=>'post_type_not_exists',	'errmsg'=>'post_type 未定义']);
			}
		}

		if(empty($post_id)){
			if($post_type == 'any'){
				wpjam_send_json(['errcode'=>'empty_post_id',	'errmsg'=>'文章ID不能为空']);
			}

			$orderby	= wpjam_get_parameter('orderby');

			if($orderby == 'rand'){
				$wp->set_query_var('orderby', 'rand');
			}else{
				$name_key	= is_post_type_hierarchical($post_type) ? 'pagename' : 'name';

				$wp->set_query_var($name_key,	wpjam_get_parameter($name_key,	['required'=>true]));
			}
		}else{
			$wp->set_query_var('p', $post_id);
		}

		$wp->set_query_var('post_type', $post_type);
		$wp->set_query_var('posts_per_page', 1);
		$wp->set_query_var('cache_results', true);

		$wp->query_posts();

		if(!$wp_query->have_posts()){
			if($post_name = get_query_var('name')){
				if($post_id = apply_filters('old_slug_redirect_post_id', null)){
					$post_type	= 'any';

					$wp->set_query_var('post_type', $post_type);
					$wp->set_query_var('posts_per_page', 1);
					$wp->set_query_var('p', $post_id);
					$wp->set_query_var('name', '');
					$wp->set_query_var('pagename', '');

					$wp->query_posts();
				}else{
					wpjam_send_json(['errcode'=>'empty_query',	'errmsg'=>'查询结果为空']);
				}
			}else{
				wpjam_send_json(['errcode'=>'empty_query',	'errmsg'=>'查询结果为空']);
			}
		}

		$wp_query->the_post();

		$_post	= wpjam_get_post(get_the_ID(), $this->args);

		$post_json	= [];

		$post_json['page_title']	= html_entity_decode(wp_get_document_title());

		foreach(['share_title'=>'page_title', 'share_image'=>null] as $key => $fallback){
			$value	= wpjam_array_pull($_post, $key);

			if($value){
				$post_json[$key]	= $value;
			}elseif($fallback){
				$post_json[$key]	= $post_json[$fallback];
			}
		}

		$output	= $this->output ?: $_post['post_type'];

		$post_json[$output]	= $_post;

		return $post_json;
	}

	public function parse_media(){
		require_once ABSPATH . 'wp-admin/includes/file.php';
		require_once ABSPATH . 'wp-admin/includes/media.php';
		require_once ABSPATH . 'wp-admin/includes/image.php';

		$media_id	= $this->media ?: 'media';
		$output		= $this->output ?: 'url';

		if (!isset($_FILES[$media_id])) {
			wpjam_send_json(['errcode'=>'empty_media',	'errmsg'=>'媒体流不能为空！']);
		}

		if($this->type == 'post_type'){
			$post_id		= (int)wpjam_get_parameter('post_id',	['method'=>'POST', 'default'=>0]);
			$attachment_id	= media_handle_upload($media_id, $post_id);

			if(is_wp_error($attachment_id)){
				wpjam_send_json($attachment_id);
			}

			$url	= wp_get_attachment_url($attachment_id);
		}else{
			$upload_file	= wp_handle_upload($_FILES[$media_id], ['test_form'=>false]);

			if(isset($upload_file['error'])){
				wpjam_send_json(['errcode'=>'upload_error',	'errmsg'=>$upload_file['error']]);
			}

			$url	= $upload_file['url'];
		}

		return [$output => $url];
	}

	public function parse_taxonomy(){
		$taxonomy	= $this->taxonomy;
		$tax_obj	= $taxonomy ? get_taxonomy($taxonomy) : null;

		if(empty($tax_obj)){
			wpjam_send_json(['errcode'=>'invalid_taxonomy',	'errmsg'=>'无效的自定义分类']);
		}

		$args	= $this->args;

		if($mapping = wpjam_array_pull($args, 'mapping')){
			$mapping	= wp_parse_args($mapping);

			if($mapping && is_array($mapping)){
				foreach($mapping as $key => $get){
					if($value = wpjam_get_parameter($get)){
						$args[$key]	= $value;
					}
				}
			}
		}

		$number		= (int)wpjam_array_pull($args, 'number');
		$output		= wpjam_array_pull($args, 'output');
		$output		= $output ?: wpjam_get_taxonomy_setting($taxonomy, 'plural');
		$max_depth	= wpjam_array_pull($args, 'max_depth');

		$terms_json	= [];

		if($terms = wpjam_get_terms($args, $max_depth)){
			if($number){
				$paged	= $args['paged'] ?? 1;
				$offset	= $number * ($paged-1);

				$terms_json['current_page']	= (int)$paged;
				$terms_json['total_pages']	= ceil(count($terms)/$number);
				$terms = array_slice($terms, $offset, $number);
			}

			$terms_json[$output]	= array_values($terms);
		}else{
			$terms_json[$output]	= [];
		}

		$terms_json['page_title']	= $tax_obj->label;

		return $terms_json;
	}

	public function parse_setting(){
		if(!$this->option_name){
			return null;
		}

		$option_name	= $this->option_name;
		$setting_name	= $this->setting_name ?? ($this->setting ?? '');

		if($this->output){
			$output	= $this->output;
		}else{
			$output	= $setting_name ? $setting_name : $option_name;
		}

		if($object = WPJAM_Option_Setting::get($option_name)){
			$value	= $object->prepare();

			if($object->option_type == 'single'){
				$value	= $value[$option_name] ?? null;

				return [$output=>$value];
			}
		}else{
			$value	= wpjam_get_option($option_name);
		}

		if($setting_name){
			$value	= $value[$setting_name] ?? null;
		}

		return [$output => $value];
	}

	public function parse_token(){
		$appid	= wpjam_get_parameter('appid',	['required'=>true]);
		$secret	= wpjam_get_parameter('secret', ['required'=>true]);
		$token	= $GLOBALS['wpjam_grant']->reset_token($appid, $secret);

		return is_wp_error($token) ? $token : ['access_token'=>$token, 'expires_in'=>7200];
	}
}

class WPJAM_Grant{
	protected $_items = [];

	public function __construct(){
		$this->init();
	}

	public function init(){
		$this->_items	= [];

		$items	= get_option('wpjam_grant');
		$items	= is_array($items) ? $items : [];

		if(isset($items['appid'])){
			$items	= [$items];
		}

		foreach($items as $item){
			if($item && is_array($item) && isset($item['appid'])){
				$this->_items[$item['appid']]	= $item;
			}
		}
	}

	public function __get($key){
		if($key == 'items'){
			return $this->_items;
		}
	}

	public function __set($key, $value){
		if($key == 'items'){
			if(update_option('wpjam_grant', array_values($value))){
				$this->init();
			}
		}
	}

	public function __call($method, $args){
		if($method == 'get_items'){
			return $this->items;
		}elseif($method == 'add'){
			if(count($this->items) >= 3){
				return new WP_Error('appid_over_quota', '最多可以设置三个 AppID');
			}

			do{
				$appid	= 'jam'.strtolower(wp_generate_password(15, false, false));
			}while(isset($this->items[$appid]));

			$this->items	= array_merge($this->items, [$appid=>compact('appid')]);

			return $appid;
		}elseif($method == 'validate_token'){
			$token	= $args[0];

			foreach($this->items as $item){
				if(isset($item['token']) && $item['token'] == $token && (time()-$item['time'] < 7200)){
					return $item;
				}
			}

			return new WP_Error('invalid_access_token', '无效的 Access Token');
		}else{
			$appid	= $args[0];
			$item	= $this->items[$appid] ?? [];

			if(!$item){
				return new WP_Error('invalid_appid', '无效的 AppID');
			}

			if($method == 'delete'){
				$this->items	= wpjam_array_except($this->items, $appid);

				return true;
			}elseif($method == 'reset_secret'){
				$secret	= strtolower(wp_generate_password(32, false, false));

				$item['secret']	= md5($secret);
				$this->items	= array_merge($this->items, [$appid=>wpjam_array_except($item, ['token', 'time'])]);

				return $secret;
			}elseif($method == 'reset_token'){
				$secret	= $args[1];

				if(empty($item['secret']) || $item['secret'] != md5($secret)){
					return new WP_Error('invalid_secret', '无效的密钥');
				}

				$item['token']	= $token = wp_generate_password(64, false, false);
				$item['time']	= time();

				$this->items	= array_merge($this->items, [$appid=>$item]);

				return $token;
			}
		}
	}
}

class WPJAM_Parameter{
	private $name;
	private $args;

	public function __construct($name, $args=[]){
		$this->name	= $name;
		$this->args	= $args;
	}

	public function __get($name){
		return $this->args[$name] ?? null;
	}

	public function __isset($name){
		return isset($this->args[$name]);
	}

	public function get_value($type=''){
		$value	= null;

		if($type == 'data'){
			if(isset($_GET[$this->name])){
				$value	= wp_unslash($_GET[$this->name]);
			}else{
				$data	= $this->get_data();
				$value	= $data[$this->name] ?? null;
			}
		}else{
			$method	= $this->method ? strtoupper($this->method) : 'GET';

			if($method == 'GET'){
				if(isset($_GET[$this->name])){
					$value	= wp_unslash($_GET[$this->name]);
				}
			}elseif($method == 'POST'){
				if(empty($_POST)){
					$input	= $this->get_input();
					$value	= $input[$this->name] ?? null;
				}elseif(isset($_POST[$this->name])){
					$value	= wp_unslash($_POST[$this->name]);
				}
			}else{
				if(isset($_REQUEST[$this->name])){
					$value	= wp_unslash($_REQUEST[$this->name]);
				}elseif(empty($_POST)){
					$input	= $this->get_input();
					$value	= $input[$this->name] ?? null;
				}
			}
		}

		if(is_null($value) && isset($this->default)){
			return $this->default;
		}

		$value	= $this->validate_value($value);

		if(is_wp_error($value)){
			if($this->send === false){
				return $value;
			}else{
				wpjam_send_json($value);
			}
		}

		return $this->sanitize_value($value);
	}

	public function validate_value($value){
		if($this->validate_callback){
			if(is_callable($this->validate_callback)){
				$result	= call_user_func($this->validate_callback, $value);

				if($result === false){
					return new WP_Error('invalid_parameter', '无效参数：'.$this->name);
				}elseif(is_wp_error($result)){
					return $result;
				}
			}
		}else{
			if($this->required){
				if(is_null($value)){
					return new WP_Error('missing_parameter', '缺少参数：'.$this->name);
				}
			}

			if($this->length){
				if(is_numeric($this->length) && mb_strlen($value) < $this->length){
					return new WP_Error('short_parameter', $this->name.' 参数长度不能少于 '.$this->length);
				}
			}
		}

		return $value;
	}

	public function sanitize_value($value){
		if($this->sanitize_callback){
			if(is_callable($this->sanitize_callback)){
				$value	= call_user_func($this->sanitize_callback, $value);
			}
		}else{
			if($this->type == 'int' && !is_null($value)){
				$value	= (int)$value;
			}
		}

		return $value;
	}

	private static $input; 
	private static $data;

	public static function get_input($name=''){
		if(is_null(self::$input)){
			$input	= file_get_contents('php://input');

			if(is_string($input)){
				$input	= @wpjam_json_decode($input);
			}

			self::$input	= is_array($input) ? $input : [];
		}

		return self::$input;
	}

	public static function get_data(){
		if(is_null(self::$data)){
			self::$data	= wpjam_get_parameter('data', ['method'=>'POST', 'sanitize_callback'=>'wp_parse_args', 'default'=>[]]);
			self::$data	= wpjam_array_merge(self::get_defaults(), self::$data);
		}

		return self::$data;
	}

	public static function get_defaults(){
		return wpjam_get_parameter('defaults', ['method'=>'POST', 'sanitize_callback'=>'wp_parse_args', 'default'=>[]]);
	}
}

class WPJAM_Request{
	private $url;
	private $args;

	public function __construct($url, $args=[]){
		if(wpjam_doing_debug()){
			print_r($url);
			print_r($args);
		}

		$this->url	= $url;
		$this->args = wp_parse_args($args, [
			'body'			=> [],
			'headers'		=> [],
			'sslverify'		=> false,
			'blocking'		=> true,	// 如果不需要立刻知道结果，可以设置为 false
			'stream'		=> false,	// 如果是保存远程的文件，这里需要设置为 true
			'filename'		=> null,	// 设置保存下来文件的路径和名字
			// 'headers'	=> ['Accept-Encoding'=>'gzip;'],	//使用压缩传输数据
			// 'compress'	=> false,
		]);

		if($this->method){
			$this->method	= strtoupper($this->method);
		}else{
			$this->method	= $this->body ? 'POST' : 'GET';
		}
	}

	public function __get($name){
		return $this->args[$name] ?? null;
	}

	public function __isset($name){
		return isset($this->args[$name]);
	}

	public function __set($name, $value){
		$this->args[$name]	= $value;
	}

	public function request($err_args=[]){
		if($this->method == 'GET'){
			$response	= wp_remote_get($this->url, $this->args);		
		}elseif($this->method == 'FILE'){	// 上传文件
			$this->method				= $this->body ? 'POST' : 'GET';
			$this->sslcertificates		= $this->sslcertificates ?? ABSPATH.WPINC.'/certificates/ca-bundle.crt';
			$this->args['user-agent']	= $this->args['user-agent'] ?? 'WordPress';

			$response	= (new WP_Http_Curl())->request($this->url, $this->args);
		}else{
			$json_encode_required	= wpjam_array_pull($this->args, ['json_encode_required', 'need_json_encode'], false);

			if($json_encode_required){
				if(is_array($this->body)){
					$this->body	= wpjam_json_encode($this->body);
				}

				if($this->method == 'POST' && empty($this->headers['Content-Type'])){
					$this->headers	+= ['Content-Type'=>'application/json'];
				}
			}

			$response	= wp_remote_request($this->url, $this->args);
		}

		if(wpjam_doing_debug()){
			print_r($response);
		}

		if(is_wp_error($response)){
			trigger_error($this->url."\n".$response->get_error_code().' : '.$response->get_error_message()."\n".var_export($this->body,true));
			return $response;
		}

		if(!empty($response['response']['code']) && $response['response']['code'] != 200){
			return new WP_Error($response['response']['code'], '远程服务器错误：'.$response['response']['code'].' - '.$response['response']['message']);
		}

		if(!$this->blocking){
			return true;
		}

		$response	= $this->decode($response['body'], $response['headers']);
		$wp_error	= $this->is_error($response, $err_args);

		return $wp_error ?: $response;
	}

	public function decode($response, $headers){
		if(empty($headers['content-disposition']) || strpos($headers['content-disposition'], 'attachment;') === false){
			$json_decode_required	= wpjam_array_pull($this->args, ['json_decode_required', 'need_json_decode'], ($this->stream ? false : true));

			if(!$json_decode_required){
				$content_type	= $headers['content-type'] ?? '';
				$content_type	= is_array($content_type) ? implode(' ', $content_type) : $content_type;

				$json_decode_required	= $content_type && strpos($content_type, '/json');
			}

			if($json_decode_required){
				if($this->stream){
					$response	= file_get_contents($this->filename);
				}

				if(empty($response)){
					trigger_error(var_export($response, true).var_export($headers, true));
				}else{
					$response	= wpjam_json_decode($response);
				}
			}
		}

		return $response;
	}

	public function is_error($response, $err_args=[]){
		if(is_wp_error($response)){
			return $response;
		}

		$err_args	= wp_parse_args($err_args,  [
			'errcode'	=>'errcode',
			'errmsg'	=>'errmsg',
			'detail'	=>'detail',
			'success'	=>'0',
		]);

		if(isset($response[$err_args['errcode']]) && $response[$err_args['errcode']] != $err_args['success']){
			$errcode	= wpjam_array_pull($response, $err_args['errcode']);
			$errmsg		= wpjam_array_pull($response, $err_args['errmsg']);
			$detail		= wpjam_array_pull($response, $err_args['detail']);
			$detail		= is_null($detail) ? array_filter($response) : $detail;

			if(apply_filters('wpjam_http_response_error_debug', true, $errcode, $errmsg, $detail)){
				trigger_error($this->url."\n".$errcode.' : '.$errmsg."\n".($detail ? var_export($detail,true)."\n" : '').var_export($this->body,true));
			}

			return new WP_Error($errcode, $errmsg, $detail);
		}

		return false;
	}
}
