<?php
class WPJAM_Platform extends WPJAM_Register{
	public function verify(){
		return call_user_func($this->verify);
	}

	public static function get_sorted(){
		return wpjam_sort_items(self::get_registereds(), 'order', 'ASC');
	}

	public static function get_options($type='bit'){
		$objects	= [];

		foreach(self::get_sorted() as $key => $object){
			if(!empty($object->bit)){
				$object->key	= $key;

				$objects[$object->bit]	= $object;
			}
		}

		if($type == 'key' || $type == 'name'){
			return wp_list_pluck($objects, 'title', 'key');
		}elseif($type == 'bit'){
			return wp_list_pluck($objects, 'title');
		}else{
			return wp_list_pluck($objects, 'bit');
		}
	}

	public static function get_current($platforms=[], $type='bit'){
		foreach(self::get_sorted() as $name => $object){
			if($object->verify()){
				$return	= $type == 'bit' ? $object->bit : $name;

				if(($platforms && in_array($return, $platforms)) 
					|| empty($platforms))
				{
					return $return;
				}
			}	
		}

		return '';
	}
}

class WPJAM_Path extends WPJAM_Register{
	private $types	= [];

	public function add_type($type, $item){
		$page_type	= $item['page_type'] ?? '';

		if($page_type 
			&& in_array($page_type, ['post_type', 'taxonomy'])
			&& empty($item[$page_type])
		){
			$item[$page_type]	= $this->name;
		}

		if(isset($item['group']) && is_array($item['group'])){
			if(isset($item['group']['key'], $item['group']['title'])){
				$group_key		= $item['group']['key'];
				$group_title	= $item['group']['title'];

				$item['group']	= $group_key;

				wpjam_register('path_group', $group_key, ['title'=>$group_title]);
			}else{
				unset($item['group']);
			}
		}

		$item['path_type']	= $type;

		$this->types[$type]	= $item;

		$this->args	= $this->args+$item;

		if(!in_array($type, self::$platforms)){
			self::$platforms[]	= $type;
		}
	}

	public function remove_type($type){
		unset($this->types[$type]);
	}

	public function get_type($type){
		return $this->types[$type] ?? [];
	}

	public function get_tabbar($type){
		$item	= $this->get_type($type);
		
		if($item && !empty($item['tabbar'])){
			$tabbar	= $item['tabbar'];

			if(!is_array($tabbar)){
				$tabbar	= ['text'=>$this->title];
			}

			return $tabbar;
		}

		return '';
	}

	public function get_raw_path($type){
		$item	= $this->get_type($type);

		return $item ? ($item['path'] ?? '') : '';
	}

	public function get_path($type, $args=[], $postfix='', $postfix_title=''){
		if($postfix){
			foreach($this->get_fields() as $field_key => $path_field){
				$_args[$field_key]	= $args[$field_key.$postfix] ?? '';
			}

			$args	= $_args;
		}

		$item	= $this->get_type($type);

		if($item){
			$callback	= wpjam_array_pull($item, 'callback');

			if(!$callback && !empty($item['page_type'])){
				$object	= wpjam_get_registered_object('page_type', $item['page_type']);

				if($object){
					$callback = [$object->model, 'get_path'];
				}
			}

			if($callback && is_callable($callback)){
				$args	= array_filter($args, 'is_exists');
				$args	= wp_parse_args($args, $item);

				return call_user_func($callback, $args, $this->name) ?: '';
			}
			
			if(isset($item['path'])){
				return $item['path'] ?: '';
			}
		}

		return new WP_Error('invalid_'.$postfix.'page_key', '无效的'.$postfix_title.'页面');
	}

	public function get_fields(){
		$fields	= [];

		foreach($this->types as $type => $item){
			$item_fields	= wpjam_array_pull($item, 'fields') ?: [];

			if(!$item_fields && !empty($item['page_type'])){
				$object	= wpjam_get_registered_object('page_type', $item['page_type']);

				if($object){
					$item_fields	= [$object->model, 'get_fields'];
				}
			}

			if($item_fields && is_callable($item_fields)){ 
				$item_fields	= call_user_func($item_fields, $item, $this->name);
			}

			if(is_array($item_fields)){
				$fields	= array_merge($fields, $item_fields);
			}
		}

		return $fields;
	}

	public function has($types, $operator='AND', $strict=false){
		$types	= (array)$types;

		foreach($types as $type){
			if($item = $this->get_type($type)){
				$has	= isset($item['path']) || isset($item['callback']);

				if($strict && $has && isset($item['path']) && $item['path'] === false){
					$has	= false;
				}
			}else{
				$has	= false;
			}

			if($operator == 'AND'){
				if(!$has){
					return false;
				}
			}elseif($operator == 'OR'){
				if($has){
					return true;
				}
			}
		}

		if($operator == 'AND'){
			return true;
		}elseif($operator == 'OR'){
			return false;
		}
	}

	private static $platforms	= [];

	public static function get_platforms(){
		return self::$platforms;
	}

	public static function parse_item($item, $path_type, $backup=false){
		if($backup){
			$postfix	= '_backup';
			$default	= 'none';
			$title		= '备用';
		}else{
			$postfix	= '';
			$default	= '';
			$title		= '';
		}

		$page_key	= $item['page_key'.$postfix] ?? '';
		$page_key	= $page_key ?: $default;
		$parsed		= [];

		if($page_key == 'none'){
			if(!empty($item['video'])){
				$parsed['type']		= 'video';
				$parsed['video']	= $item['video'];
				$parsed['vid']		= wpjam_get_qqv_id($item['video']);
			}else{
				$parsed['type']		= 'none';
			}
		}elseif($page_key == 'external'){
			if(in_array($path_type, ['web', 'template'])){
				$parsed['type']		= 'external';
				$parsed['url']		= $item['url'];
			}
		}elseif($page_key == 'web_view'){
			if(in_array($path_type, ['web', 'template'])){
				$parsed['type']		= 'external';
				$parsed['url']		= $item['src'];
			}else{
				$parsed['type']		= 'web_view';
				$parsed['src']		= $item['src'];
			}
		}

		if(!$parsed && $page_key){
			$path_obj	= self::get($page_key);

			if($path_obj){
				$path	= $path_obj->get_path($path_type, $item, $postfix, $title);

				if(!is_wp_error($path)){
					if(is_array($path)){
						$parsed	= $path;
					}else{
						$parsed['type']		= '';
						$parsed['page_key']	= $page_key;
						$parsed['path']		= $path;
					}
				}
			}
		}

		return $parsed;
	}

	public static function validate_item($item, $path_types, $backup=false){
		if($backup){
			$postfix	= '_backup';
			$default	= 'none';
			$title		= '备用';
		}else{
			$postfix	= '';
			$default	= '';
			$title		= '';
		}

		$page_key	= $item['page_key'.$postfix] ?: $default;

		if($page_key == 'none'){
			return true;
		}elseif($page_key == 'web_view'){
			if(!$backup){
				$path_types	= array_diff($path_types, ['web','template']);
			}
		}

		$path_obj	= self::get($page_key);

		if($path_obj){
			foreach($path_types as $path_type){
				$path	= $path_obj->get_path($path_type, $item, $postfix, $title);

				if(is_wp_error($path)){
					return $path;
				}
			}
		}else{
			return new WP_Error('invalid_'.$postfix.'page_key', $title.'页面无效');
		}

		return true;
	}

	public static function parse_page_key_options(&$options, $path_obj){
		$group_title	= '其他页面';
		$group_key		= 'others';

		if($path_obj->tabbar){
			$group_title	= '菜单栏	/常用';
			$group_key		= 'tabbar';
		}elseif($path_obj->group){
			$object	= wpjam_get_registered_object('path_group', $path_obj->group);

			if($object){
				$group_title	= $object->title;
				$group_key		= $path_obj->group;
			}
		}

		if(!isset($options[$group_key])){
			$options[$group_key]	= ['title'=>$group_title, 'options'=>[]];
		}

		$options[$group_key]['options'][$path_obj->name]	= $path_obj->title;
	}
	
	public static function get_path_fields($path_types, $for=''){
		$strict		= ($for == 'qrcode');

		$backup_required	= count($path_types) > 1 && !$strict;

		if($backup_required){
			$backup_fields	= ['page_key_backup'=>['title'=>'',	'type'=>'select',	'options'=>[],	'description'=>'跳转页面不生效时将启用备用页面']];
			$show_if_keys	= [];
		}

		$page_key_fields	= ['page_key'=>['title'=>'',	'type'=>'select',	'options'=>[]]];

		foreach(self::get_registereds() as $page_key => $path_obj){
			if(!$path_obj->has($path_types, 'OR', $strict)){
				continue;
			}

			self::parse_page_key_options($page_key_fields['page_key']['options'], $path_obj);

			$path_fields	= $path_obj->get_fields();

			foreach($path_fields as $field_key => $path_field){
				if(isset($path_field['show_if'])){
					$page_key_fields[$field_key]	= $path_field;
				}else{
					if(isset($page_key_fields[$field_key])){
						$page_key_fields[$field_key]['show_if']['value'][]	= $page_key;
					}else{
						$path_field['title']	= '';
						$path_field['show_if']	= ['key'=>'page_key','compare'=>'IN','value'=>[$page_key]];

						$page_key_fields[$field_key]	= $path_field;
					}
				}
			}

			if($backup_required){
				if($path_obj->has($path_types, 'AND')){
					if(
						($page_key != 'module_page' && empty($path_fields)) 
						|| ($page_key == 'module_page' && $path_fields)
					){
						self::parse_page_key_options($backup_fields['page_key_backup']['options'], $path_obj);
					}

					if($page_key == 'module_page' && $path_fields){
						foreach($path_fields as $field_key => $path_field){
							$path_field['show_if']	= ['key'=>'page_key_backup','value'=>$page_key];
							$backup_fields[$field_key.'_backup']	= $path_field;
						}
					}
				}else{
					if($page_key == 'web_view'){
						if(!$path_obj->has(array_diff($path_types, ['web','template']), 'AND')){
							$show_if_keys[]	= $page_key;
						}
					}else{
						$show_if_keys[]	= $page_key;
					}
				}
			}
		}

		if($for == 'qrcode'){
			return ['page_key_set'=>['title'=>'页面',	'type'=>'fieldset',	'fields'=>$page_key_fields]];
		}else{
			$page_key_fields['page_key']['options']['tabbar']['options']['none']	= '只展示不跳转';

			$fields	= [];

			$fields['page_key_set']	= ['title'=>'页面',	'type'=>'fieldset',	'fields'=>$page_key_fields];

			if($backup_required){
				$show_if	= ['key'=>'page_key','compare'=>'IN','value'=>$show_if_keys];

				$backup_fields['page_key_backup']['options']['tabbar']['options']['none']	= '只展示不跳转';

				$fields['page_key_backup_set']	= ['title'=>'备用',	'type'=>'fieldset',	'fields'=>$backup_fields, 'show_if'=>$show_if];
			}

			return $fields;
		}
	}

	public static function get_by(...$args){
		if(is_array($args[0])){
			$args	= $args[0];
		}else{
			$args	= [$args[0] => $args[1]];
		}

		$path_type	= wpjam_array_pull($args, 'path_type');
		$path_objs	= self::get_registereds($args);

		if($path_type){
			$path_objs	= array_filter($path_objs, function($path_obj) use($path_type){
				return $path_obj->has($path_type);
			});
		}
		
		return $path_objs;
	}

	public static function create($page_key, $args=[]){
		wpjam_register_path($page_key, $args);
	}
}