<?php
function wpjam_register($group, $name, $args=[], $type=''){
	return WPJAM_Register::register_by_group($group, $name, $args, $type);
}

function wpjam_unregister($group, $name, $args=[], $type=''){
	WPJAM_Register::unregister_by_group($group, $name, $args, $type);
}

function wpjam_get_registereds($group, $type=''){
	return WPJAM_Register::get_by_group($group, $type);
}

function wpjam_get_registered_object($group, $name){
	return WPJAM_Register::get($name, $group);
}

function wpjam_register_route_module($name, $args){
	return wpjam_register('route_module', $name, $args);
}

function wpjam_is_module($module='', $action=''){
	$_module	= wpjam_get_current_module();
	$_action	= wpjam_get_current_action();

	if($module){
		if($action && $action != $_action){
			return false;
		}

		return $module == $_module;
	}elseif($_module){
		return true;
	}else{
		return false;
	}
}

function wpjam_get_current_module(){
	return $GLOBALS['wp']->query_vars['module'] ?? null;
}

function wpjam_get_current_action(){
	return $GLOBALS['wp']->query_vars['action'] ?? null;
}

function wpjam_register_json($name, $args=[]){
	if(WPJAM_JSON::get($name)){
		trigger_error('API 「'.$name.'」已经注册。');
	}

	return WPJAM_JSON::register($name, $args);
}

function wpjam_register_api($name, $args=[]){
	return wpjam_register_json($name, $args);
}

function wpjam_get_json_object($name){
	return WPJAM_JSON::get($name);
}

function wpjam_get_json(){
	return wpjam_get_current_json();
}

function wpjam_get_current_json(){
	if(wpjam_get_current_module() == 'json'){
		return WPJAM_JSON::parse_name(wpjam_get_current_action());
	}

	return '';
}

function wpjam_is_json_request(){
	if(get_option('permalink_structure')){
		if(preg_match("/\/api\/(.*)\.json/", $_SERVER['REQUEST_URI'])){ 
			return true;
		}
	}else{
		if(isset($_GET['module']) && $_GET['module'] == 'json'){
			return true;
		}
	}

	return false;
}


function wpjam_get_current_user($required=false){
	$user	= apply_filters('wpjam_current_user', null);

	if($required){
		if(is_null($user)){
			return new WP_Error('bad_authentication', '无权限');
		}
	}else{
		if(is_wp_error($user)){
			return null;
		}
	}

	return $user;
}

function wpjam_get_current_commenter(){
	if(get_option('comment_registration')){
		return new WP_Error('logged_in_required', '只支持登录用户操作');
	}

	$commenter	= wp_get_current_commenter();

	if(empty($commenter['comment_author_email'])){
		return new WP_Error('bad_authentication', '无权限');
	}

	return $commenter;
}

// 注册后台选项
function wpjam_register_option($name, $args=[]){
	$args	= is_callable($args) ? call_user_func($args, $name) : $args;
	$args	= apply_filters('wpjam_register_option_args', $args, $name);

	return WPJAM_Option_Setting::register($name, $args);
}

function wpjam_unregister_option($name){
	WPJAM_Option_Setting::unregister($name);
}

function wpjam_get_option_object($name){
	return WPJAM_Option_Setting::get($name);
}


// 添加后台菜单
function wpjam_add_menu_page($menu_slug, $args=[]){
	if(is_admin()){
		if(!empty($args['menu_title'])){
			WPJAM_Menu_Page::add($menu_slug, $args);
		}
	}else{
		if(isset($args['function']) && $args['function'] == 'option'){
			if(!empty($args['sections']) || !empty($args['fields'])){
				$option_name	= $args['option_name'] ?? $menu_slug;

				wpjam_register_option($option_name, $args);
			}
		}
	}
}

// 注册 Meta 类型
function wpjam_register_meta_type($name, $args=[]){
	$name	= sanitize_key($name);
	$object	= WPJAM_Meta_Type::register($name, $args);

	$table_name	= $object->table_name;

	if(!isset($GLOBALS['wpdb']->$table_name)){
		$GLOBALS['wpdb']->$table_name = $object->get_table();
	}

	wpjam_register_lazyloader($name.'_meta', [
		'filter'	=> 'get_'.$name.'_metadata',
		'callback'	=> [$object, 'update_cache']
	]);

	return $object;
}

function wpjam_get_meta_type_object($name){
	return WPJAM_Meta_Type::get($name);
}

function wpjam_get_by_meta($meta_type, ...$args){
	if(empty($args)){
		return [];
	}

	$object	= wpjam_get_meta_type_object($meta_type);

	if($object){
		if(is_array($args[0])){
			$meta_key	= wpjam_array_get($args[0], ['meta_key', 'key']);
			$meta_value	= wpjam_array_get($args[0], ['meta_value', 'value']);
		}else{
			$meta_key	= $args[0];
			$meta_value	= $args[1] ?? null;
		}

		return $object->get_by_key($meta_key, $meta_value);
	}

	return [];
}

// wpjam_get_metadata($meta_type, $object_id, $meta_keys)
// wpjam_get_metadata($meta_type, $object_id, $meta_key, $default)
function wpjam_get_metadata($meta_type, $object_id, ...$args){
	$object	= wpjam_get_meta_type_object($meta_type);

	if(is_array($args[0])){
		$data	= [];

		if($object_id && $object){
			foreach($args[0] as $meta_key => $default){
				if(is_numeric($meta_key)){
					$meta_key	= $default;
					$default	= null;
				}

				$data[$meta_key]	= $object->get_data_with_default($object_id, $meta_key, $default);
			}	
		}

		return $data;
	}else{
		$meta_key	= $args[0];
		$default	= $args[1] ?? null;

		if($object_id && $object){
			return $object->get_data_with_default($object_id, $meta_key, $default);
		}

		return $default;
	}

	return [];
}

// wpjam_update_metadata($meta_type, $object_id, $data, $defaults=[])
// wpjam_update_metadata($meta_type, $object_id, $meta_key, $meta_value, $default=null)
function wpjam_update_metadata($meta_type, $object_id, ...$args){
	$object	= wpjam_get_meta_type_object($meta_type);

	if(!$object){
		return null;
	}

	if(is_array($args[0])){
		$data		= $args[0];
		$defaults	= (isset($args[1]) && is_array($args[1])) ? $args[1] : array_keys($data);

		foreach($defaults as $meta_key => $default){
			if(is_numeric($meta_key)){
				if(is_numeric($default)){
					continue;
				}

				$meta_key	= $default;
				$default	= null;
			}

			$meta_value	= $data[$meta_key] ?? null;

			$object->update_data_with_default($object_id, $meta_key, $meta_value, $default);
		}

		return true;
	}else{
		$meta_key	= $args[0];
		$meta_value	= $args[1];
		$default	= $args[2] ?? null;

		return $object->update_data_with_default($object_id, $meta_key, $meta_value, $default);
	}
}



// 注册 LazyLoader
function wpjam_register_lazyloader($name, $args){
	$object	= WPJAM_Lazyloader::get($name);

	return $object ?: WPJAM_Lazyloader::register($name, $args);
}

function wpjam_get_lazyloader($name){
	return WPJAM_Lazyloader::get($name);
}

function wpjam_lazyload($name, $ids, ...$args){
	$object	= WPJAM_Lazyloader::get($name);

	if($object){
		$object->queue_objects($ids, ...$args);
	}
}


// 注册文章类型
function wpjam_register_post_type($name, $args=[]){
	return WPJAM_Post_Type::register($name, $args);
}

function wpjam_add_post_type_field($post_type, $key, $field){
	$object	= WPJAM_Post_Type::get($post_type);

	return $object ? $object->add_field($key, $field) : null;
}

function wpjam_remove_post_type_field($post_type, $key){
	$object	= WPJAM_Post_Type::get($post_type);

	return $object ? $object->remove_field($key, $field) : null;
}

function wpjam_get_post_type_fields($post_type){
	$object	= WPJAM_Post_Type::get($post_type);

	return $object ? $object->get_fields() : [];
}

function wpjam_get_post_type_setting($post_type, $key=null, $default=null){
	$object	= WPJAM_Post_Type::get($post_type);

	return $object ? $object->get_arg($key, $default) : ($key ? $default : []);
}

function wpjam_update_post_type_setting($post_type, ...$args){
	$object	= WPJAM_Post_Type::get($post_type);

	if($object){
		if(is_array($args[0])){
			foreach($args[0] as $key => $value){
				$object->update_arg($key, $value);
			}
		}else{
			$object->update_arg($args[0], $args[1]);
		}

		return true;
	}
}


// 注册文章选项
function wpjam_register_post_option($meta_box, $args=[]){
	if(WPJAM_Post_Option::get($meta_box)){
		trigger_error('Post Option 「'.$meta_box.'」已经注册。');
	}

	return WPJAM_Post_Option::register($meta_box, $args);
}

function wpjam_unregister_post_option($meta_box){
	WPJAM_Post_Option::unregister($meta_box);
}

function wpjam_register_posts_column($name, ...$args){
	if(is_admin()){
		$field	= is_array($args[0]) ? $args[0] : ['title'=>$args[0], 'column_callback'=>($args[1] ?? null)];

		return wpjam_register_list_table_column($name, array_merge($field, ['data_type'=>'post_type']));
	}
}

function wpjam_unregister_posts_column($name){
	if(is_admin()){
		add_filter('manage_'.get_current_screen()->post_type.'_posts_columns', function($columns) use($name){
			return wpjam_array_except($columns, $name);
		}, 9999);
	}
}


// 注册分类模式
function wpjam_register_taxonomy($name, $args=[]){
	return WPJAM_Taxonomy::register($name, $args);
}

function wpjam_add_taxonomy_field($taxonomy, $key, $field){
	$object	= WPJAM_Taxonomy::get($taxonomy);

	return $object ? $object->add_field($key, $field) : null;
}

function wpjam_remove_taxonomy_field($taxonomy, $key){
	$object	= WPJAM_Taxonomy::get($taxonomy);

	return $object ? $object->remove_field($key, $field) : null;
}

function wpjam_get_taxonomy_fields($taxonomy){
	$object	= WPJAM_Taxonomy::get($taxonomy);

	return $object ? $object->get_fields() : [];
}

function wpjam_get_taxonomy_setting($taxonomy, $key=null, $default=null){
	$object	= WPJAM_Taxonomy::get($taxonomy);

	return $object ? $object->get_arg($key, $default) : ($key ? $default : []);
}

function wpjam_update_taxonomy_setting($taxonomy, ...$args){
	$object	= WPJAM_Taxonomy::get($taxonomy);

	if($object){
		if(is_array($args[0])){
			foreach($args[0] as $key => $value){
				$object->update_arg($key, $value);
			}
		}else{
			$object->update_arg($args[0], $args[1]);
		}

		return true;
	}
}



// 注册分类选项
function wpjam_register_term_option($name, $args=[]){
	if(WPJAM_Term_Option::get($name)){
		trigger_error('Term Option 「'.$name.'」已经注册。');
	}

	if(is_callable($args)){
		trigger_error('callable_term_option_args');
		$args	= ['fields'=>$fields];
	}elseif(!isset($args['fields'])){
		$args['fields']	= [$name => wpjam_array_except($args, 'taxonomy')];
	}

	return WPJAM_Term_Option::register($name, $args);
}

function wpjam_unregister_term_option($name){
	WPJAM_Term_Option::unregister($name);
}

function wpjam_register_terms_column($name, ...$args){
	if(is_admin()){
		$field	= is_array($args[0]) ? $args[0] : ['title'=>$args[0], 'column_callback'=>($args[1] ?? null)];

		return wpjam_register_list_table_column($name, array_merge($field, ['data_type'=>'taxonomy']));
	}
}

function wpjam_unregister_terms_column($name){
	if(is_admin()){
		add_filter('manage_'.get_current_screen()->id.'_columns', function($columns) use($name){
			return wpjam_array_except($columns, $name);
		}, 9999);
	}
}

// 注册绑定
function wpjam_register_bind($name, $appid, $model){
	if($instance = wpjam_get_bind_type_object($name, $appid)){
		return $instance;
	}

	return WPJAM_Bind_Type::register($name.':'.$appid, [
		'name'		=> $name, 
		'appid'		=> $appid,
		'object'	=> is_object($model) ? $model:  new $model($appid)
	]);
}

function wpjam_get_bind_type_object($name, $appid){
	return WPJAM_Bind_Type::get($name.':'.$appid);
}

function wpjam_get_bind_object($name, $appid){
	$instance	= wpjam_get_bind_type_object($name, $appid);

	return $instance ? $instance->object : null;
}

// 注册登录方式
function wpjam_register_user_signup($name, $args){
	WPJAM_User_Signup_Type::register($name, $args);
}

function wpjam_get_user_signups($args=[], $output='objects', $operator='and'){
	return WPJAM_User_Signup_Type::get_registereds($args, $output, $operator);
}

function wpjam_get_user_signup_object($name){
	return WPJAM_User_Signup_Type::get($name);
}


// 注册 AJAX
function wpjam_register_ajax($name, $args){
	$object	= wpjam_get_ajax_object($name);
	$object	= $object ?: WPJAM_AJAX::register($name, $args);

	add_action('wp_ajax_'.$name, [$object, 'callback']);

	if($object->nopriv){
		add_action('wp_ajax_nopriv_'.$name, [$object, 'callback']);
	}

	return $object;
}

function wpjam_get_ajax_object($name){
	return WPJAM_AJAX::get($name);
}

function wpjam_get_ajax_attributes($name, $data=[]){
	if($object = wpjam_get_ajax_object($name)){
		return $object->get_attributes($data);
	}

	return [];
}

function wpjam_get_ajax_attribute_string($name, $data=[]){
	return wpjam_data_attribute_string(wpjam_get_ajax_attributes($name, $data));
}

function wpjam_ajax_enqueue_scripts(){
	WPJAM_AJAX::enqueue_scripts();
}


// 注册平台
function wpjam_register_platform($name, $args){
	return WPJAM_Platform::register($name, $args);
}

function wpjam_is_platform($platform){
	return WPJAM_Platform::get($platform)->verify();
}

function wpjam_get_current_platform($platforms=[], $type='key'){
	return WPJAM_Platform::get_current($platforms, $type);
}

function wpjam_get_platform_options($type='bit'){
	return WPJAM_Platform::get_options($type);
}

function wpjam_get_current_platforms(){
	return WPJAM_Path::get_platforms();
}

// 注册路径
function wpjam_register_path($page_key, ...$args){
	if(count($args) == 2){
		$item	= $args[1]+['path_type'=>$args[0]];
		$args	= [$item];
	}else{
		$args	= $args[0];
		$args	= wp_is_numeric_array($args) ? $args : [$args];
	}

	$object	= WPJAM_Path::get($page_key);
	$object	= $object ?: WPJAM_Path::register($page_key, []);

	foreach($args as $item){
		$type	= wpjam_array_pull($item, 'path_type');

		$object->add_type($type, $item);
	}

	return $object;
}

function wpjam_unregister_path($page_key, $path_type=''){
	if($path_type){
		if($object = WPJAM_Path::get($page_key)){
			$object->remove_type($path_type);
		}
	}else{
		WPJAM_Path::unregister($page_key);
	}
}

function wpjam_get_path_object($page_key){
	return WPJAM_Path::get($page_key);
}

function wpjam_get_paths($path_type){
	return WPJAM_Path::get_by(['path_type'=>$path_type]);
}

function wpjam_get_tabbar_options($path_type){
	$options	= [];
	
	foreach(WPJAM_Path::get_registereds() as $page_key => $object){
		$tabbar	= $object->get_tabbar($path_type);

		if($tabbar){
			$options[$page_key]	= $tabbar['text'];
		}
	}

	return $options;
}

function wpjam_get_path_fields($path_types, $for=''){
	if(empty($path_types)){
		return [];
	}

	$path_types	= (array)$path_types;

	return WPJAM_Path::get_path_fields($path_types, $for);
}

function wpjam_get_page_keys($path_type){
	$pages	= [];

	foreach(WPJAM_Path::get_registereds() as $page_key => $object){
		if($path = $object->get_raw_path($path_type)){
			$pos		= strrpos($path, '?');
			$pages[]	= [
				'page_key'	=> $page_key, 
				'page'		=> $pos ? substr($path, 0, $pos) : $path,
			];
		}
	}

	return $pages;
}

// wpjam_get_path($path_type, $page_key, $args);
// wpjam_get_path($path_type, $args);
function wpjam_get_path($path_type, ...$args){
	if(is_array($args[0])){
		$args		= $args[0];
		$page_key	= wpjam_array_pull($args, 'page_key');
	}else{
		$page_key	= $args[0];
		$args		= $args[1] ?? [];
	}

	$object	= wpjam_get_path_obj($page_key);

	return $object ? $object->get_path($path_type, $args) : '';
}

function wpjam_parse_path_item($item, $path_type, $parse_backup=true){
	$parsed	= WPJAM_Path::parse_item($item, $path_type);

	if(empty($parsed) && $parse_backup && !empty($item['page_key_backup'])){
		$parsed	= WPJAM_Path::parse_item($item, $path_type, true);
	}

	return $parsed ?: ['type'=>'none'];
}

function wpjam_validate_path_item($item, $path_types){
	$result	= WPJAM_Path::validate_item($item, $path_types);

	if(is_wp_error($result)){
		if(count($path_types) > 1 && $result->get_error_code() == 'invalid_page_key'){
			return WPJAM_Path::validate_item($item, $path_types, true);	
		}
	}

	return $result;
}

function wpjam_get_path_item_link_tag($parsed, $text){
	if($parsed['type'] == 'none'){
		return $text;
	}elseif($parsed['type'] == 'external'){
		return '<a href_type="web_view" href="'.$parsed['url'].'">'.$text.'</a>';
	}elseif($parsed['type'] == 'web_view'){
		return '<a href_type="web_view" href="'.$parsed['src'].'">'.$text.'</a>';
	}elseif($parsed['type'] == 'mini_program'){
		return '<a href_type="mini_program" href="'.$parsed['path'].'" appid="'.$parsed['appid'].'">'.$text.'</a>';
	}elseif($parsed['type'] == 'contact'){
		return '<a href_type="contact" href="" tips="'.$parsed['tips'].'">'.$text.'</a>';
	}elseif($parsed['type'] == ''){
		return '<a href_type="path" page_key="'.$parsed['page_key'].'" href="'.$parsed['path'].'">'.$text.'</a>';
	}
}

function wpjam_register_page_type($page_type, $args){
	return wpjam_register('page_type', $page_type, $args);
}

function wpjam_register_theme_upgrader($upgrader_url){
	$object	= wpjam_register('theme_upgrader', get_template(), ['upgrader_url'=>$upgrader_url]);

	add_filter('site_transient_update_themes',	'wpjam_site_transient_update_themes');
}

function wpjam_site_transient_update_themes($transient){
	$theme	= get_template();
	$object	= wpjam_get_registered_object('theme_upgrader', $theme);

	if($object && $object->upgrader_url){
		if(empty($transient->checked[$theme])){
			return $transient;
		}

		$remote	= get_transient('wpjam_theme_upgrade_'.$theme);

		if(false == $remote){
			$remote = wpjam_remote_request($object->upgrader_url);

			if(!is_wp_error($remote)){
				set_transient('wpjam_theme_upgrade_'.$theme, $remote, HOUR_IN_SECONDS*12);
			}
		}

		if($remote && !is_wp_error($remote)){
			if(version_compare($transient->checked[$theme], $remote['new_version'], '<')){
				$transient->response[$theme]	= $remote;
			}
		}
	}

	return $transient;
}

function wpjam_register_verify_txt($name, $args){
	return WPJAM_Verify_TXT::register($name, $args);
}

function wpjam_register_capability($capability, $map_meta_cap){
	$object	= wpjam_register('capability', $capability, ['map_meta_cap'=>$map_meta_cap]);

	if(!has_filter('map_meta_cap', 'wpjam_map_meta_cap')){
		add_filter('map_meta_cap', 'wpjam_map_meta_cap', 10, 4);
	}

	return $object;
}

function wpjam_map_meta_cap($caps, $cap, $user_id, $args){
	if(!in_array('do_not_allow', $caps) && $user_id){
		if($object = wpjam_get_registered_object('capability', $cap)){
			return call_user_func($object->map_meta_cap, $user_id, $args, $cap);
		}
	}

	return $caps;
}

function wpjam_register_cron($hook, $args=[]){
	if(is_callable($hook)){
		return wpjam_register_job($hook, $args);
	}

	$cron	= WPJAM_Cron::get($hook);

	if($cron){
		return $cron;
	}

	$args	= wp_parse_args($args, ['recurrence'=>'', 'time'=>time(),	'args'=>[]]);
	$cron	= WPJAM_Cron::register($hook, $args);

	return $cron->schedule();
}

function wpjam_register_job($name, $args=[]){
	if(is_numeric($args)){
		$args	= ['weight'=>$args];
	}elseif(!is_array($args)){
		$args	= [];
	}

	if(empty($args['callback']) || !is_callable($args['callback'])){
		if(is_callable($name)){
			$args['callback']	= $name;

			if(is_object($name)){
				$name	= get_class($name);
			}elseif(is_array($name)){
				$name	= implode(':', $name);
			}
		}else{
			return null;
		}
	}

	return wpjam_register('job', $name, wp_parse_args($args, ['weight'=>1, 'day'=>-1]));
}

function wpjam_get_jobs($raw=false){
	$jobs	= [];
	$day	= (current_time('H') > 2 && current_time('H') < 6) ? 0 : 1;

	foreach(wpjam_get_registereds('job') as $name => $object){
		if($raw || $object->day == -1 || $object->day == $day){
			$jobs[$name]	= $object->to_array();
		}
	}

	return $jobs;
}

function wpjam_is_scheduled_event($hook) {	// 不用判断参数
	$wp_crons	= _get_cron_array() ?: [];

	foreach($wp_crons as $timestamp => $cron){
		if(isset($cron[$hook])){
			return true;
		}
	}

	return false;
}

function wpjam_generate_verification_code($key, $group='default'){
	if($object = WPJAM_Verification_Code::get($group)){
		return $object->generate($key);
	}

	return new WP_Error('invalid_verification_code_group','无效的验证码分组');
}

function wpjam_verify_code($key, $code, $group='default'){
	if($object = WPJAM_Verification_Code::get($group)){
		return $object->verify($key, $code);
	}

	return new WP_Error('invalid_verification_code_group','无效的验证码分组');
}

function wpjam_register_verification_code_group($name, $args=[]){
	return WPJAM_Verification_Code::register($name, $args);
}

function wpjam_register_field_data_type($name, $args=[]){
	return WPJAM_Field_Data_Type::register($name, $args);
}

function wpjam_get_data_type_field($name, $args){
	if($object = WPJAM_Field_Data_Type::get($name)){
		return $object->get_field($args) ?: [];
	}

	return [];
}

function wpjam_register_gravatar_services($name, $args){
	return wpjam_register('gravatar', $name, $args);
}

function wpjam_register_google_font_services($name, $args){
	return wpjam_register('google_font', $name, $args);
}

add_action('wpjam_loaded',	function(){
	wpjam_register_platform('weapp',	['bit'=>1,	'order'=>4,		'title'=>'小程序',	'verify'=>'is_weapp']);
	wpjam_register_platform('weixin',	['bit'=>2,	'order'=>4,		'title'=>'微信网页',	'verify'=>'is_weixin']);
	wpjam_register_platform('mobile',	['bit'=>4,	'order'=>8,		'title'=>'移动网页',	'verify'=>'wp_is_mobile']);
	wpjam_register_platform('web',		['bit'=>8,	'order'=>10,	'title'=>'网页',		'verify'=>'__return_true']);
	wpjam_register_platform('template',	['bit'=>8,	'order'=>10,	'title'=>'网页',		'verify'=>'__return_true']);

	wpjam_register_page_type('post_type',	['model'=>'WPJAM_Post_Type_Page_Type']);
	wpjam_register_page_type('taxonomy',	['model'=>'WPJAM_Taxonomy_Page_Type']);
	wpjam_register_page_type('author',		['model'=>'WPJAM_Author_Page_Type']);

	wpjam_register_field_data_type('post_type', ['model'=>'WPJAM_Post_Type_Data_Type']);
	wpjam_register_field_data_type('taxonomy',	['model'=>'WPJAM_Taxonomy_Data_Type']);
	wpjam_register_field_data_type('model',		['model'=>'WPJAM_Model_Data_Type']);
	wpjam_register_field_data_type('video',		['model'=>'WPJAM_Video_Data_Type']);

	wpjam_register_meta_type('post');
	wpjam_register_meta_type('term');
	wpjam_register_meta_type('user');
	wpjam_register_meta_type('comment');

	if(is_multisite()){
		wpjam_register_meta_type('blog');
		wpjam_register_meta_type('site');
	}

	wpjam_register_lazyloader('user',		['filter'=>'wpjam_get_userdata',	'callback'=>'cache_users']);
	wpjam_register_lazyloader('post_term',	['filter'=>'loop_start',			'callback'=>'update_object_term_cache',	'accepted_args'=>2]);

	wpjam_register_gravatar_services('cravatar',	['title'=>'Cravatar',	'url'=>'https://cravatar.cn/avatar/']);
	wpjam_register_gravatar_services('geekzu',		['title'=>'极客族',		'url'=>'https://sdn.geekzu.org/avatar/']);
	wpjam_register_gravatar_services('loli',		['title'=>'loli',		'url'=>'https://gravatar.loli.net/avatar/']);
	wpjam_register_gravatar_services('loli',		['title'=>'loli',		'url'=>'https://gravatar.loli.net/avatar/']);
	wpjam_register_gravatar_services('sep_cc',		['title'=>'sep.cc',		'url'=>'https://cdn.sep.cc/avatar/']);

	wpjam_register_google_font_services('geekzu',	['title'=>'极客族',		'replace'=>[
		'//fonts.geekzu.org',
		'//gapis.geekzu.org/ajax',
		'//gapis.geekzu.org/g-themes',
		'//gapis.geekzu.org/g-fonts'
	]]);

	wpjam_register_google_font_services('loli',		['title'=>'loli',		'replace'=>[
		'//fonts.loli.net',
		'//ajax.loli.net',
		'//themes.loli.net',
		'//gstatic.loli.net'
	]]);

	wpjam_register_google_font_services('ustc',		['title'=>'中科大',		'replace'=>[
		'//fonts.lug.ustc.edu.cn',
		'//ajax.lug.ustc.edu.cn',
		'//google-themes.lug.ustc.edu.cn',
		'//fonts-gstatic.lug.ustc.edu.cn'
	]]);

	wpjam_register_verification_code_group('default');
});

add_action('init',	['WPJAM_Route', 'init']);
add_action('init',	['WPJAM_Post_Type', 'init']);
add_action('init',	['WPJAM_Taxonomy', 'init']);
add_filter('init',	['WPJAM_Cron', 'init']);

add_action('parse_request',	['WPJAM_Route', 'on_parse_request']);
add_action('send_headers',	['WPJAM_Route', 'on_send_headers']);

add_filter('determine_current_user',	['WPJAM_Route', 'filter_determine_current_user']);
add_filter('wp_get_current_commenter',	['WPJAM_Route', 'filter_current_commenter']);

add_filter('register_post_type_args',	['WPJAM_Post_Type', 'filter_register_args'], 999, 2);
add_filter('register_taxonomy_args',	['WPJAM_Taxonomy', 'filter_register_args'], 999, 3);

if(version_compare( $GLOBALS['wp_version'], '6.0', '<' )){
	add_action('registered_post_type',	['WPJAM_Post_Type', 'on_registered'], 1, 2);
	add_action('registered_taxonomy',	['WPJAM_Taxonomy', 'on_registered'], 1, 3);
}

if(wpjam_is_json_request()){
	remove_filter('the_title', 'convert_chars');

	remove_action('init', 'wp_widgets_init', 1);
	remove_action('init', 'maybe_add_existing_user_to_blog');
	remove_action('init', 'check_theme_switched', 99);

	remove_action('plugins_loaded', '_wp_customize_include');

	remove_action('wp_loaded', '_custom_header_background_just_in_time');
}
